#!/usr/local/bin/perl

# Greg Stitt
# This takes the instruction power file as input and formats the data so
# that it can be displayed using xdu

(scalar @ARGV) == 2 or die "Usage: format_pwr.pl <input_file> <output_file>\n";

$ARITH_INS = 0;
$LOGIC_INS = 1;
$TRANSFER_INS = 2;
$BOOL_INS = 3;
$BRANCH_INS = 4;

@categories = ($ARITH_INS, $LOGIC_INS, $TRANSFER_INS, $BOOL_INS, $BRANCH_INS);
@catg_names = 
    ("ARITH_INS", "LOGIC_INS", "TRANSFER_INS", "BOOL_INS", "BRANCH_INS");

#categorize instructions

$ins_table{"ACALL"} = $BRANCH_INS;
$ins_table{"ADD_1"} = $ARITH_INS;
$ins_table{"ADD_2"} = $ARITH_INS;
$ins_table{"ADD_3"} = $ARITH_INS;
$ins_table{"ADD_4"} = $ARITH_INS;
$ins_table{"ADDC_1"} = $ARITH_INS;
$ins_table{"ADDC_2"} = $ARITH_INS;
$ins_table{"ADDC_3"} = $ARITH_INS;
$ins_table{"ADDC_4"} = $ARITH_INS;
$ins_table{"AJMP"} = $BRANCH_INS;
$ins_table{"ANL_1"} = $LOGIC_INS;
$ins_table{"ANL_2"} = $LOGIC_INS;
$ins_table{"ANL_3"} = $LOGIC_INS;
$ins_table{"ANL_4"} = $LOGIC_INS;
$ins_table{"ANL_5"} = $LOGIC_INS;
$ins_table{"ANL_6"} = $LOGIC_INS;
$ins_table{"ANL_7"} = $BOOL_INS;
$ins_table{"ANL_8"} = $BOOL_INS;
$ins_table{"CJNE_1"} = $BRANCH_INS;
$ins_table{"CJNE_2"} = $BRANCH_INS;
$ins_table{"CJNE_3"} = $BRANCH_INS;
$ins_table{"CJNE_4"} = $BRANCH_INS;
$ins_table{"CLR_1"} = $LOGIC_INS;
$ins_table{"CLR_2"} = $BOOL_INS;
$ins_table{"CLR_3"} = $BOOL_INS;
$ins_table{"CPL_1"} = $LOGIC_INS;
$ins_table{"CPL_2"} = $BOOL_INS;
$ins_table{"CPL_3"} = $BOOL_INS;
$ins_table{"DA"} = $ARITH_INS;
$ins_table{"DEC_1"} = $ARITH_INS;
$ins_table{"DEC_2"} = $ARITH_INS;
$ins_table{"DEC_3"} = $ARITH_INS;
$ins_table{"DEC_4"} = $ARITH_INS;
$ins_table{"DIV"} = $ARITH_INS;
$ins_table{"DJNZ_1"} = $BRANCH_INS;
$ins_table{"DJNZ_2"} = $BRANCH_INS;
$ins_table{"INC_1"} = $ARITH_INS;
$ins_table{"INC_2"} = $ARITH_INS;
$ins_table{"INC_3"} = $ARITH_INS;
$ins_table{"INC_4"} = $ARITH_INS;
$ins_table{"INC_5"} = $ARITH_INS;
$ins_table{"JB"} = $BOOL_INS;
$ins_table{"JBC"} = $BOOL_INS;
$ins_table{"JC"} = $BOOL_INS;
$ins_table{"JMP"} = $BRANCH_INS;
$ins_table{"JNB"} = $BOOL_INS;
$ins_table{"JNC"} = $BOOL_INS;
$ins_table{"JNZ"} = $BRANCH_INS;
$ins_table{"JZ"} = $BRANCH_INS;
$ins_table{"LCALL"} = $BRANCH_INS;
$ins_table{"LJMP"} = $BRANCH_INS;
$ins_table{"MOV_1"} = $TRANSFER_INS;
$ins_table{"MOV_2"} = $TRANSFER_INS;
$ins_table{"MOV_3"} = $TRANSFER_INS;
$ins_table{"MOV_4"} = $TRANSFER_INS;
$ins_table{"MOV_5"} = $TRANSFER_INS;
$ins_table{"MOV_6"} = $TRANSFER_INS;
$ins_table{"MOV_7"} = $TRANSFER_INS;
$ins_table{"MOV_8"} = $TRANSFER_INS;
$ins_table{"MOV_9"} = $TRANSFER_INS;
$ins_table{"MOV_10"} = $TRANSFER_INS;
$ins_table{"MOV_11"} = $TRANSFER_INS;
$ins_table{"MOV_12"} = $TRANSFER_INS;
$ins_table{"MOV_13"} = $TRANSFER_INS;
$ins_table{"MOV_14"} = $TRANSFER_INS;
$ins_table{"MOV_15"} = $TRANSFER_INS;
$ins_table{"MOV_16"} = $BOOL_INS;
$ins_table{"MOV_17"} = $BOOL_INS;
$ins_table{"MOV_18"} = $TRANSFER_INS;
$ins_table{"MOVC_1"} = $TRANSFER_INS;
$ins_table{"MOVC_2"} = $TRANSFER_INS;
$ins_table{"MOVX_1"} = $TRANSFER_INS;
$ins_table{"MOVX_2"} = $TRANSFER_INS;
$ins_table{"MOVX_3"} = $TRANSFER_INS;
$ins_table{"MOVX_4"} = $TRANSFER_INS;
$ins_table{"MUL"} = $ARITH_INS;
$ins_table{"NOP"} = $BRANCH_INS;
$ins_table{"ORL_1"} = $LOGIC_INS;
$ins_table{"ORL_2"} = $LOGIC_INS;
$ins_table{"ORL_3"} = $LOGIC_INS;
$ins_table{"ORL_4"} = $LOGIC_INS;
$ins_table{"ORL_5"} = $LOGIC_INS;
$ins_table{"ORL_6"} = $LOGIC_INS;
$ins_table{"ORL_7"} = $BOOL_INS;
$ins_table{"ORL_8"} = $BOOL_INS;
$ins_table{"POP"} = $TRANSFER_INS;
$ins_table{"PUSH"} = $TRANSFER_INS;
$ins_table{"RET"} = $BRANCH_INS;
$ins_table{"RETI"} = $BRANCH_INS;
$ins_table{"RL"} = $LOGIC_INS;
$ins_table{"RLC"} = $LOGIC_INS;
$ins_table{"RR"} = $LOGIC_INS;
$ins_table{"RRC"} = $LOGIC_INS;
$ins_table{"SETB_1"} = $BOOL_INS;
$ins_table{"SETB_2"} = $BOOL_INS;
$ins_table{"SJMP"} = $BRANCH_INS;
$ins_table{"SUBB_1"} = $ARITH_INS;
$ins_table{"SUBB_2"} = $ARITH_INS;
$ins_table{"SUBB_3"} = $ARITH_INS;
$ins_table{"SUBB_4"} = $ARITH_INS;
$ins_table{"SWAP"} = $LOGIC_INS;
$ins_table{"XCH_1"} = $TRANSFER_INS;
$ins_table{"XCH_2"} = $TRANSFER_INS;
$ins_table{"XCH_3"} = $TRANSFER_INS;
$ins_table{"XCHD"} = $TRANSFER_INS;
$ins_table{"XRL_1"} = $LOGIC_INS;
$ins_table{"XRL_2"} = $LOGIC_INS;
$ins_table{"XRL_3"} = $LOGIC_INS;
$ins_table{"XRL_4"} = $LOGIC_INS;
$ins_table{"XRL_5"} = $LOGIC_INS;
$ins_table{"XRL_6"} = $LOGIC_INS;

open OUTFILE, ">$ARGV[1]" or die "Can't open $ARGV[1] as output file.\n";

foreach $catg (@categories) {
    open INFILE, "$ARGV[0]" or die "Can't open $ARGV[0] as input file.\n";
    
    while (<INFILE>) {
	($ins, $pow) = split;

	$pow = int ($pow * 100);
  
	if ($ins_table{$ins} == $catg) {
	    print OUTFILE "$pow\t./$catg_names[$catg]/$ins\n";
	    $catg_pwr[$catg] += $pow;
	}
    }
    
    print OUTFILE "$catg_pwr[$catg]\t./$catg_names[$catg]\n";
    close INFILE;
}

$total_pow = 0;

foreach $pow (@catg_pwr) {
    $total_pow += $pow
}

print OUTFILE "$total_pow\t./\n";
close OUTFILE;











