//-----------------------------------------------------------------------------

// Roman Lysecky, Tony Givargis, and Greg Stitt
// Copyright 1999, All Rights Reserved.

//-----------------------------------------------------------------------------

// Version 1.2

//-----------------------------------------------------------------------------

#ifndef i8051_inc
#define i8051_inc

//-----------------------------------------------------------------------------

#include <iostream>
#include <list>
#include "i8051_const.h"
#include "pmempwr.h"
#include "i8051ram.h"
#include "block.h"

//-----------------------------------------------------------------------------

//#define DEBUG
//#define DEBUG_PC
//#define DETAIL
//#define PORTS
//#define PROGRAM_COMPLETION ((unsigned char)RAM[P0] == 0xF5)

 //-----------------------------------------------------------------------------

class I8051 
{
  private:
    enum Flag { P, USER, OV, RS0, RS1, F0, AC, CY };

  private:
    const int InvalidData;
    const int LineLength;
    const int RecordTypeLength;
    
  private:
    const unsigned short ACC;
    const unsigned short PSW;
    const unsigned short B;
    const unsigned short SP;
    const unsigned short P0;
    const unsigned short P1;
    const unsigned short P2;
    const unsigned short P3;
    const unsigned short DPL;
    const unsigned short DPH;

  private:
    char ROM[RomSize];
    I8051Ram RAM;
    unsigned short PC;
    unsigned long instrCount;
    unsigned long cycleCount;
    unsigned char IR;
    unsigned short tempDPTR;
    bool progEnd;
    unsigned progSize;
  
  private:
    list<block*> blockList;

  private:
    PmemPwr pmem_power;

  private:
    char trace_out[MAX_FILE_NAME];
    char insfreq_out[MAX_FILE_NAME];
    char pmempwr_out[MAX_FILE_NAME];
    char ramaccess_out[MAX_FILE_NAME];
 
  public:
    I8051();
    ~I8051();
    
  public:
    bool Simulate(const char* fileName);
    bool ProgramCompletion();
    void Stop();
    void Init8051();
    Opcode Decode(const unsigned char IR);
    void SetOutput(const char* trace, const char* freq, const char* pmem,
                 const char* ramaccess);
  

  private:
    void SetBit(char &thisByte, unsigned char thisBit);
    void ClearBit(char &thisByte, unsigned char thisBit);
    unsigned char GetBit(char thisByte, unsigned char thisBit);
    unsigned char GetRegisterBank();

  private:
    void PrintHex(unsigned char byte, ostream* os = &cout);
    void PrintPorts();
    
  private:
    bool Hex2Short(const char* buf, unsigned &val);
    bool Load(const char* buf, unsigned char* rom, unsigned& prgSize);
    bool LoadHex(const char *filename);

  private:
    void FindBlocks();
    void TallyBlock();
    void PrintBlockInfo(char* infile);
  
};

//-----------------------------------------------------------------------------

#endif

