
#include <fstream.h>
#include <iomanip.h>
#include <string.h>
#include "i8051ram.h"



I8051Ram::I8051Ram()
{
  memset(RAM, 0, sizeof(RamAddrStat) * RamSize);
  start_log = false;

  for (unsigned int i=0; i < RamSize; i++) RamLocNames[i] = "";
  RamLocNames[0x00] = "RegBank0";
  RamLocNames[0x08] = "RegBank1";
  RamLocNames[0x10] = "RegBank2";
  RamLocNames[0x18] = "RegBank3";
  RamLocNames[0x20] = "BitAddrSpace";
  RamLocNames[0x30] = "LowAddrSpace";
  RamLocNames[0x80] = "IndrAddrSpace";
  RamLocNames[0x80+128] = "P0";
  RamLocNames[0x81+128] = "SP";
  RamLocNames[0x82+128] = "DPL";
  RamLocNames[0x83+128] = "DPH";
  RamLocNames[0x87+128] = "PCON";
  RamLocNames[0x88+128] = "TCON";
  RamLocNames[0x89+128] = "TMOD";
  RamLocNames[0x8a+128] = "TL0";
  RamLocNames[0x8b+128] = "TL1";
  RamLocNames[0x8c+128] = "TH0";
  RamLocNames[0x8d+128] = "TH1";
  RamLocNames[0x90+128] = "P1";
  RamLocNames[0x98+128] = "SCON";
  RamLocNames[0x99+128] = "SBUF";
  RamLocNames[0xa0+128] = "P2";
  RamLocNames[0xa8+128] = "IE";
  RamLocNames[0xb0+128] = "P3";
  RamLocNames[0xb8+128] = "IP";
  RamLocNames[0xd0+128] = "PSW";
  RamLocNames[0xe0+128] = "ACC";
  RamLocNames[0xf0+128] = "B";
}

I8051Ram:: ~I8051Ram()
{
}

bool I8051Ram::PrintRamAccess(char* output="ramaccess.out")
{
  ofstream outfile;
  outfile.open(output);
  if (!outfile) {
      cerr << "Warning: Error opening " << output 
           << " for ram access stats." << endl;
      
      return false;
  }
  
  for (unsigned int i=0; i < RamSize; i ++) {
/*      outfile << setw(5) << setfill('0') << i << "\t"
              << setw(15) << setiosflags(ios::left) << setfill(' ') 
              << RamLocNames[i].c_str()
              << "\t" << RAM[i].freq << resetiosflags(ios::left) 
              << endl;      */

      outfile << setw(5) << setfill('0') << i << "\t"
              << setw(15) << setiosflags(ios::left) << setfill(' ')
              << RAM[i].freq << "\t" << RamLocNames[i].c_str() 
              << resetiosflags(ios::left) << endl;
  }

  return true;
}

void I8051Ram::StatLogging(bool start)
{
  if (start == true) 
    start_log = true;
  else
    start_log = false;
}

char& I8051Ram::operator[](unsigned int index)
{
  if (start_log == true) RAM[index].freq ++;
  return RAM[index].data;
}












