//-----------------------------------------------------------------------------

// Roman Lysecky, Tony Givargis, and Greg Stitt
// Copyright 1999, All Rights Reserved.

//-----------------------------------------------------------------------------

// Version 1.2

//-----------------------------------------------------------------------------

#include <iostream>
#include <signal.h>
#include <unistd.h>
#include "i8051.h"

//-----------------------------------------------------------------------------

void SigHandler(int signal);

//-----------------------------------------------------------------------------

static I8051 i8051;

//-----------------------------------------------------------------------------

#define NUM_FLAGS 5

enum flag_names {
  RUN_TIME_FLAG=0,
  TRACE_FLAG,
  INSFREQ_FLAG,
  PMEMPWR_FLAG,
  RAMACCESS_FLAG
};

static char* flags[] = {
  "-r",
  "-t",
  "-i",
  "-p",
  "-a"
};

static const char* usage = 
"usage: [-r time] [-t file] [-i file] [-p file] [-a file] HEXFILE\n"
"    -r         Specifies length of time in second to run\n"
"    -t         Specifies different trace output file\n"
"    -i         Specifies output file for instruction frequencies\n"
"    -p         Specifies output file for pmem power\n"
"    -a         Specifies output file for ram access";

void usage_exit() {
  cerr << usage << endl;
  exit(0);
}

//-----------------------------------------------------------------------------

int 
main(int argc, char* argv[]) 
{     
    int runtime=0;
    int i,j;

    char hexfile[MAX_FILE_NAME];
    char tracefile[MAX_FILE_NAME];
    char freqfile[MAX_FILE_NAME];
    char pmemfile[MAX_FILE_NAME];
    char ramaccessfile[MAX_FILE_NAME];
    
    hexfile[0] = tracefile[0] = freqfile[0] = pmemfile[0] =
        ramaccessfile[0]= '\0';
    
    // process flags
    for (i=1; i < argc; i++) {
        for (j=0; j < NUM_FLAGS; j++) {
            if (!strcmp(argv[i], flags[j])) break;
        }
        
        switch (j) {
            case RUN_TIME_FLAG:
              if (i+1 == argc) usage_exit();
              runtime = atoi(argv[++i]);
              if (runtime == 0) usage_exit();
              break;

            case TRACE_FLAG:
              if (i+1 == argc) usage_exit();
              strcpy(tracefile, argv[++i]);
              break;
              
            case INSFREQ_FLAG:
              if (i+1 == argc) usage_exit();
              strcpy(freqfile, argv[++i]);
              break;

            case PMEMPWR_FLAG:
              if (i+1 == argc) usage_exit();
              strcpy(pmemfile, argv[++i]);
              break;

            case RAMACCESS_FLAG:
              if (i+1 == argc) usage_exit();
              strcpy(ramaccessfile, argv[++i]);
              break;

            default:
              if (hexfile[0] != '\0') usage_exit();
              strcpy(hexfile, argv[i]);
              break;
        }
    }
    
    if (hexfile[0] == '\0') usage_exit();

    i8051.SetOutput(tracefile[0] == '\0' ? "output.txt" : tracefile,
                    freqfile[0] == '\0' ? "insfreq.out" : freqfile,
                    pmemfile[0] == '\0' ? "pmempwr.out" : pmemfile,
                    ramaccessfile[0] == '\0' ? "ramaccess.out" : ramaccessfile);
                    
    signal(SIGINT, SigHandler);
    signal(SIGALRM, SigHandler);
    alarm(runtime);
    
    i8051.Simulate(hexfile);
  
    return(0);
}

//-----------------------------------------------------------------------------

void SigHandler(int signal) 
{
    i8051.Stop();
}

//-----------------------------------------------------------------------------
