//-------------------------------------------------------------------------

// Greg Stitt

//-------------------------------------------------------------------------
// This file handles the tracing of instruction power
//-------------------------------------------------------------------------

#include <string.h>
#include <fstream.h>
#include <iomanip.h>
#include "pmempwr.h"

PmemPwr::PmemPwr()
{
  for (unsigned int i=0; i < RomSize; i ++) {
      memset(&ROM[i], 0, sizeof(PwrStats));
  }

  memset(InsPwr, 0, sizeof(double) * NumIns);
//  memset(InsFreq, 0, sizeof(InsCountStats) * NumIns);
}

PmemPwr::~PmemPwr()
{
}

bool PmemPwr::UpdateAddr (unsigned short PC, Opcode ins) 
{
  if (PC >= RomSize)
    return false;

  ROM[PC].ins = ins;
  ROM[PC].freq ++;

  // ins is stored because InsFreq will be sorted by freq
  InsFreq[ins].ins = ins;
  InsFreq[ins].freq ++;

  return true;
}

bool PmemPwr::PrintPower (char* output="pmempwr.out")
{
  ofstream outfile;
  outfile.open(output);
  if (!outfile) {
      cerr << "Warning: Unable to open trace file " << output << "." << endl;
      return false;
  }

  if (!GetPower()) {
      outfile << "Warning: Can't open or find power input file, " 
              << "output will not be accurate." << endl;
  }
  
  for (unsigned int i=0; i < RomSize; i ++) {
      if (ROM[i].freq > 0) {
          if (ROM[i].ins < (Opcode) 0 || ROM[i].ins > (Opcode) NumIns) {
              cerr << "Warning: Invalid instruction in PmemPwr" << endl;
              return false;
          }
          
          outfile << setw(5) << setfill('0') << i
                  << "\t"  << InsNameTbl[ROM[i].ins] 
                  << "\t" << ROM[i].freq 
                  << "\t" << ROM[i].pwr 
                  << "\t" << ROM[i].freq_pwr << endl;
      }
  }

  outfile.close();
  return true;
}

bool PmemPwr::PrintFreq(char* output="insfreq.out")
{
  ofstream outfile;
  outfile.open(output);
  if (!outfile) {
      cerr << "Warning: Unable to open trace file " << output << "." << endl;
      return false;
  }
  
  sort( InsFreq, InsFreq + NumIns);
  for(unsigned int i=0; i < NumIns && InsFreq[i].freq > 0; i++) {
      outfile << InsNameTbl[InsFreq[i].ins]
              << "\t" << InsFreq[i].freq << endl;
  }

  outfile.close();
  return true;
}


bool PmemPwr::GetPower(char* input="inspower.txt")
{
  double power;
  char ins[MAX_INS_NAME];
  ifstream infile;
  
  infile.open(input);
  if (!infile) {
      cerr << "Warning: Unable to open power file " << input << "." << endl;
      return false;
  }

  // read power of each instruction into InsPwr
  while (!infile.eof()) {
      infile >> ins >> power;
      
      // linear search for instruction in table
      
      for (unsigned int i=0; i < NumIns; i ++) {
          if (!strcmp(ins, InsNameTbl[i])) {
              //cout << ins << "\t" << power << endl;
              
              InsPwr[i] = power;
          }
      }
  }
  
  infile.close();

  // set stats for each location in ROM
  for (unsigned int j=0; j < RomSize; j ++) {
      ROM[j].pwr = InsPwr[ROM[j].ins];
      ROM[j].freq_pwr = ROM[j].pwr * ROM[j].freq;
  }

  return true;
}







