#include "core_tree.h"
#include <list>
#include <string>


//#define DEBUG
//#define TOTAL_NETS_DEBUG

core_tree::core_tree()
{ 
  core *root_core = new core(ROOT_NAME);
  root.push_front(root_core);
  total = 0;
}

core_tree::~core_tree()
{
  for (list <core*>::iterator i = root.begin(); i != root.end(); i ++)
    delete *i;
}

int core_tree::insert(string core_name, double power)
{
  list <core*>::iterator i = root.begin();
  list <core*> *node = &root;
  int pos = -1;
  
  search(ROOT_NAME, i, *node);
  (*i)->power += power;
 
  while (pos != core_name.size())
  {
      pos = core_name.find(DELIMETER, pos + 1);
  
      if (pos == string::npos)
        pos = core_name.size();

#ifdef DEBUG
      cout << "POS = " << pos << endl;
#endif
            
      string component = core_name.substr(0, pos);
      
      if ((!search(component, i, *node)) && power > 0)
      {
#ifdef DEBUG
          cout << "INSERTED " << component << endl;
#endif
          core* new_core = new core(component);
          node->push_front(new_core);
          i = node->begin();
      }
#ifdef DEBUG
      else
      {
          cout << component << " found!" << endl;
      }
#endif
      
      (*i)->power += power;
      
      node = &((*i)->next);
  }

  total ++;
}
 
// search function
// returns 1 if name is found and 0 otherwise
// updates j with the location where the item was found

int core_tree::search(string name, list<core*>::iterator &j,
                      list<core*> &node)
{
  list<core*>::iterator i;

  for (i = node.begin(); i != node.end(); i ++)
  {
      if ((*i)->name == name)
      {
          j = i; 
          return 1;
      }
  }
  
  return 0;
}

void core_tree::print (int mode)
{
  switch (mode)
  {
      case NORMAL_MODE:
        print_rec(root, 0);
        break;
      case DU_MODE:
        print_du(root);
        break;
      default:
        cout << "Unknown print mode" << endl;
        break;
  }

#ifdef TOTAL_NETS_DEBUG
  cout << "Total nets = " << total << endl;
#endif
}

void core_tree::print_rec(list<core*> node, int level)
{
  list<core*>::iterator i;

  for (i = node.begin(); i != node.end(); i ++)
  {
      for (int j = 0; j < level; j++)
        cout << "\t";

      cout << (*i)->name << "\t" << (*i)->power << endl;
      print_rec((*i)->next, level + 1);
  }
}

void core_tree::print_du(list<core*> node)
{
  list<core*>::iterator i;
  for (i = node.begin(); i != node.end(); i ++)
  {
      print_du((*i)->next);
      if ((int) (*i)->power > 0)
        cout << (int) ((*i)->power) << "\t" << "./" << (*i)->name << endl;
  }
}




