#!/usr/local/bin/perl

# Greg Stitt
# this program reports all significant power data of the currect architecture

#special sorting functions 
sub ins_pwr_sort { 
    $ins_pwr{$b} <=> $ins_pwr{$a}; 
}
sub ram_frq_sort { 
    ${$b}[1] <=> ${$a}[1];
}
sub max_comp_pwr_sort { 
    ${$b}[0] <=> ${$a}[0]; 
}

#specify the components to be searched for in component power report
$components[0] = "U_CTR";
$components[1] = "U_RAM";
$components[2] = "U_DEC";
$components[3] = "U_ALU";
$components[4] = "U_ROM";

# get Version title
$num_args = scalar @ARGV;
if ($num_args != 1) {
    print "usage: $0 version_name\n";
    print "The version name is used to print a title on the report.\n";
    exit;
}


print "POWER CONSUMPTION REPORT FOR VERSION $ARGV[0]\n\n\n";

# specify number of results to report
$num_ins_frq = 10;
$num_ins_pwr = 10;
$num_ram_addr = 10;
$num_max_comp_pwr = 20;

# reports the most expensive instructions
$inspow = 0;
open INSPWR, "inspower.txt" or $inspow = 1;
if ($inspow == 0) {
    while (<INSPWR>) {
        @power_args = split;
	$ins_pwr{$power_args[0]} = $power_args[1];
    }
    
    @sortins = sort ins_pwr_sort keys %ins_pwr;
    
    $i = 0;
    print "The $num_ins_pwr most expensive instructions were:\n";
    foreach $ins (@sortins) {
	if ($i < $num_ins_pwr) {
	    print "$ins\t$ins_pwr{$ins}\n";
	}
	
	$i++;
    }
}
else {
    print "Average power per instruction could not be found.\n";
}
close INSPWR;

print "\n\n";

# this will generate a report on all the files in the iss_results directory
$iss = 0;
opendir ISSDIR, "iss_results" or $iss = 1;
@allfiles = readdir ISSDIR;
closedir ISSDIR;

if ($iss == 0) {
    
    foreach $file (@allfiles) {   
	# finds the most frequent instructions in each testbench application
	if ($file =~ /.frq/) {    
	    @ins_frq = ();
	    
	    open FRQFILE, "iss_results/$file" or die "Error opening $file\n";
	    while (<FRQFILE>) {
		push @ins_frq, [ split ];
	    }
	    close FRQFILE;
	    
	    # get filename without extension
	    @file_name = split /[.]/, $file, 2;
	    $ins_frq_hash{$file_name[0]} = [ @ins_frq ];
	}
	
	# finds the ram locations that are accessed the most 
	if ($file =~ /.ram/) {    
	    @ram_frq = ();
	    
	    open RAMFILE, "iss_results/$file" or die "Error opening $file\n";
	    while (<RAMFILE>) {
		push @ram_frq, [ split ];
	    }
	    close RAMFILE;
	    
	    @ram_frq = sort ram_frq_sort @ram_frq;
	    
	    # get filename without extension
	    @file_name = split /[.]/, $file, 2;
	    $ram_frq_hash{$file_name[0]} = [ @ram_frq ];
	}
	
    }
    
    # prints the most frequent instructions in a two dimensional array 
    print "The $num_ins_frq most frequent instructions were:\n";
    foreach $file (@allfiles) {
	if ($file =~ /.frq/) {
	    # get filename without extension
	    @file_name = split /[.]/, $file, 2;
	    print "$file_name[0]\t\t";
	}
    }
    print "\n";
    for $i (0 .. $num_ins_frq - 1) {
	foreach $key (keys %ins_frq_hash) {
            if (defined ($ins_frq_hash{$key}[$i])) {
		print join ( "\t", @{$ins_frq_hash{$key}[$i]}), "\t"; 
	    }
	    else {
                # handles case when there are more than instructions
		# requested than actually in rom
		print "\t\t";
	    }
	}
	print "\n";
    }
    
    # prints the most frequent ram accesses in a 2 dimensional array
    print "\n\nThe $num_ram_addr most frequently accessed locations in ram were:\n";
    foreach $file (@allfiles) {
	if ($file =~ /.ram/) {
	    # get filename without extension
	    @file_name = split /[.]/, $file, 2;
	    print "$file_name[0]\t\t";
	}
    }
    print "\n";
    for $i (0 .. $num_ram_addr - 1) {
	foreach $key (keys %ram_frq_hash) {
            if (defined ($ram_frq_hash{$key}[$i])) {
		#print join ( "\t", @{$ram_frq_hash{$key}[$i]}), "\t";
		print "$ram_frq_hash{$key}[$i][0]\t$ram_frq_hash{$key}[$i][1]\t";
	    }
	}
	print "\n";
    } 
}
else {
    print "ISS results could not be found.\n";
}


# this reports all data in the ph_results directory
$ph_results = 0;
opendir PHRESULTS, "ph_results" or $ph_results = 1;
@allfiles = readdir PHRESULTS;
closedir PHRESULTS;

if ($ph_results == 0) {
    
    # this calculates the power of the major components and the power of the
    # the most expensive components/nets
    foreach $file (@allfiles) {   
	if ($file =~ /.xdu/) {    
	    @comp_pwr = ();
	    @max_comp_pwr = ();
	    
	    # get filename without extension
	    @file_name = split /[.]/, $file, 2;
	    
	    open XDUFILE, "ph_results/$file" or die "Error opening $file\n";
	    while (<XDUFILE>) {
		@tmp_pwr = split;
		
		push @max_comp_pwr, [ @tmp_pwr ];
		
		# finds the power for the specified components
		foreach $comp (@components) {
		    if ($tmp_pwr[1] =~ /$comp$/) {
			$comp_pwr_hash{$file_name[0]}{$comp} = $tmp_pwr[0];
		    }
		}
	    }
	    close XDUFILE;

	    # sorts the power of all components
	    @max_comp_pwr = sort max_comp_pwr_sort @max_comp_pwr;
	    $max_comp_pwr_hash{$file_name[0]} = [ @max_comp_pwr ];		
	}
    }

    # prints the power of specified components\n";
    print "\n\nThe power consumption of the major components was: \n";
    foreach $file (@allfiles) {
	if ($file =~ /.xdu/) {
	    @file_name = split /[.]/, $file, 2;
	    print "\t$file_name[0]";
	}
    }
    print "\n";
    foreach $comp (@components) {
	print "$comp\t";
	foreach $key (keys %comp_pwr_hash) {
	    print "$comp_pwr_hash{$key}{$comp}\t";
	}
	print "\n";
    }

    # handles the components that consume the most power
    print "\n\nThe $num_max_comp_pwr most expensive components/nets were:\n";
    foreach $file (@allfiles) {
	if ($file =~ /.xdu/) {
	    # get filename without extension
	    @file_name = split /[.]/, $file, 2;
	    print "$file_name[0]\t\t";
	}
    }
    print "\n";
    for $i (0 .. $num_max_comp_pwr - 1) {
	foreach $key (keys %max_comp_pwr_hash) {
            if (defined ($max_comp_pwr_hash{$key}[$i])) {
		print join ( "\t", @{$max_comp_pwr_hash{$key}[$i]}), "\t";
	    }
	}
	print "\n";
    } 
}
else {
    print "Power heirarchy results could not be found\n";
}

exit;












